@@ -40,6 +40,7 @@ struct connman_device {
 	connman_bool_t disconnected;
 	connman_bool_t reconnect;
 	connman_uint16_t scan_interval;
+	connman_uint16_t backoff_interval;
 	char *name;
 	char *node;
 	char *address;
@@ -57,6 +58,8 @@ struct connman_device {
 	GHashTable *networks;
 };
 
+#define SCAN_INITIAL_DELAY 10
+
 static gboolean device_scan_trigger(gpointer user_data)
 {
 	struct connman_device *device = user_data;
@@ -87,9 +90,23 @@ static void reset_scan_trigger(struct connman_device *device)
 	clear_scan_trigger(device);
 
 	if (device->scan_interval > 0) {
-		guint interval = device->scan_interval;
+		guint interval;
+
+		if (g_hash_table_size(device->networks) == 0) {
+			if (device->backoff_interval >= device->scan_interval)
+				device->backoff_interval = SCAN_INITIAL_DELAY;
+			interval = device->backoff_interval;
+		} else
+			interval = device->scan_interval;
+
+		DBG("interval %d", interval);
+
 		device->scan_timeout = g_timeout_add_seconds(interval,
 					device_scan_trigger, device);
+
+		device->backoff_interval *= 2;
+		if (device->backoff_interval > device->scan_interval)
+			device->backoff_interval = device->scan_interval;
 	}
 }
 
@@ -188,6 +205,7 @@ enum connman_service_type __connman_device_get_service_type(struct connman_devic
 int __connman_device_enable(struct connman_device *device)
 {
 	int err;
+	enum connman_service_type type;
 
 	DBG("device %p %d", device, device->blocked);
 
@@ -220,7 +238,8 @@ int __connman_device_enable(struct connman_device *device)
 	if (__connman_profile_get_offlinemode() == TRUE)
 		__connman_profile_set_offlinemode(FALSE, FALSE);
 
-	__connman_technology_enable_device(device);
+	type = __connman_device_get_service_type(device);
+	__connman_technology_enable(type);
 
 	return 0;
 }
@@ -228,6 +247,7 @@ int __connman_device_enable(struct connman_device *device)
 int __connman_device_disable(struct connman_device *device)
 {
 	int err;
+	enum connman_service_type type;
 
 	DBG("device %p", device);
 
@@ -258,7 +278,8 @@ int __connman_device_disable(struct connman_device *device)
 	device->powered_pending = FALSE;
 	device->powered = FALSE;
 
-	__connman_technology_disable_device(device);
+	type = __connman_device_get_service_type(device);
+	__connman_technology_disable(type);
 
 	return 0;
 }
@@ -474,6 +495,7 @@ struct connman_device *connman_device_create(const char *node,
 
 	service_type = __connman_device_get_service_type(device);
 	device->blocked = __connman_technology_get_blocked(service_type);
+	device->backoff_interval = SCAN_INITIAL_DELAY;
 
 	switch (type) {
 	case CONNMAN_DEVICE_TYPE_UNKNOWN:
@@ -636,6 +658,7 @@ int connman_device_set_powered(struct connman_device *device,
 						connman_bool_t powered)
 {
 	int err;
+	enum connman_service_type type;
 
 	DBG("driver %p powered %d", device, powered);
 
@@ -655,10 +678,12 @@ int connman_device_set_powered(struct connman_device *device,
 	device->powered = powered;
 	device->powered_pending = powered;
 
+	type = __connman_device_get_service_type(device);
+
 	if (device->powered == TRUE)
-		__connman_technology_enable_device(device);
+		__connman_technology_enable(type);
 	else
-		__connman_technology_disable_device(device);
+		__connman_technology_disable(type);
 
 	if (device->offlinemode == TRUE && powered == TRUE)
 		return connman_device_set_powered(device, FALSE);
@@ -1014,6 +1039,9 @@ void __connman_device_decrease_connections(struct connman_device *device)
 		return;
 
 	device->connections--;
+
+	if (device->connections == 0)
+		device->backoff_interval = SCAN_INITIAL_DELAY;
 }
 
 /**
