@@ -51,6 +51,8 @@ struct connman_ipconfig {
 	enum connman_ipconfig_method method;
 	struct connman_ipaddress *address;
 	struct connman_ipaddress *system;
+
+	int ipv6_privacy_config;
 };
 
 struct connman_ipdevice {
@@ -79,6 +81,7 @@ struct connman_ipdevice {
 	struct connman_ipconfig *config_ipv6;
 
 	gboolean ipv6_enabled;
+	int ipv6_privacy;
 };
 
 static GHashTable *ipdevice_hash = NULL;
@@ -352,6 +355,67 @@ static void set_ipv6_state(gchar *ifname, gboolean enable)
 	fclose(f);
 }
 
+static int get_ipv6_privacy(gchar *ifname)
+{
+	gchar *path;
+	FILE *f;
+	int value;
+
+	if (ifname == NULL)
+		return 0;
+
+	path = g_strdup_printf("/proc/sys/net/ipv6/conf/%s/use_tempaddr",
+								ifname);
+
+	if (path == NULL)
+		return 0;
+
+	f = fopen(path, "r");
+
+	g_free(path);
+
+	if (f == NULL)
+		return 0;
+
+	if (fscanf(f, "%d", &value) <= 0)
+		value = 0;
+
+	fclose(f);
+
+	return value;
+}
+
+/* Enable the IPv6 privacy extension for stateless address autoconfiguration.
+ * The privacy extension is described in RFC 3041 and RFC 4941
+ */
+static void set_ipv6_privacy(gchar *ifname, int value)
+{
+	gchar *path;
+	FILE *f;
+
+	if (ifname == NULL)
+		return;
+
+	path = g_strdup_printf("/proc/sys/net/ipv6/conf/%s/use_tempaddr",
+								ifname);
+
+	if (path == NULL)
+		return;
+
+	if (value < 0)
+		value = 0;
+
+	f = fopen(path, "r+");
+
+	g_free(path);
+
+	if (f == NULL)
+		return;
+
+	fprintf(f, "%d", value);
+	fclose(f);
+}
+
 static void free_ipdevice(gpointer data)
 {
 	struct connman_ipdevice *ipdevice = data;
@@ -377,6 +441,7 @@ static void free_ipdevice(gpointer data)
 	g_free(ipdevice->address);
 
 	set_ipv6_state(ipdevice->ifname, ipdevice->ipv6_enabled);
+	set_ipv6_privacy(ipdevice->ifname, ipdevice->ipv6_privacy);
 
 	g_free(ipdevice->ifname);
 	g_free(ipdevice);
@@ -474,6 +539,7 @@ void __connman_ipconfig_newlink(int index, unsigned short type,
 	ipdevice->type = type;
 
 	ipdevice->ipv6_enabled = get_ipv6_state(ipdevice->ifname);
+	ipdevice->ipv6_privacy = get_ipv6_privacy(ipdevice->ifname);
 
 	ipdevice->address = g_strdup(address);
 
@@ -682,7 +748,8 @@ void __connman_ipconfig_deladdr(int index, int family, const char *label,
 	ipdevice->address_list = g_slist_remove(ipdevice->address_list,
 								ipaddress);
 
-	connman_ipaddress_free(ipaddress);
+	connman_ipaddress_clear(ipaddress);
+	g_free(ipaddress);
 
 	connman_info("%s {del} address %s/%u label %s", ipdevice->ifname,
 						address, prefixlen, label);
@@ -852,7 +919,13 @@ void __connman_ipconfig_foreach(void (*function) (int index, void *user_data),
 	g_list_free(keys);
 }
 
-unsigned short __connman_ipconfig_get_type(int index)
+enum connman_ipconfig_type __connman_ipconfig_get_config_type(
+					struct connman_ipconfig *ipconfig)
+{
+	return ipconfig ? ipconfig->type : CONNMAN_IPCONFIG_TYPE_UNKNOWN;
+}
+
+unsigned short __connman_ipconfig_get_type_from_index(int index)
 {
 	struct connman_ipdevice *ipdevice;
 
@@ -863,7 +936,7 @@ unsigned short __connman_ipconfig_get_type(int index)
 	return ipdevice->type;
 }
 
-unsigned int __connman_ipconfig_get_flags(int index)
+unsigned int __connman_ipconfig_get_flags_from_index(int index)
 {
 	struct connman_ipdevice *ipdevice;
 
@@ -874,7 +947,7 @@ unsigned int __connman_ipconfig_get_flags(int index)
 	return ipdevice->flags;
 }
 
-const char *__connman_ipconfig_get_gateway(int index)
+const char *__connman_ipconfig_get_gateway_from_index(int index)
 {
 	struct connman_ipdevice *ipdevice;
 
@@ -904,6 +977,110 @@ void __connman_ipconfig_set_index(struct connman_ipconfig *ipconfig, int index)
 	ipconfig->index = index;
 }
 
+const char *__connman_ipconfig_get_local(struct connman_ipconfig *ipconfig)
+{
+	if (ipconfig->address == NULL)
+		return NULL;
+
+	return ipconfig->address->local;
+}
+
+void __connman_ipconfig_set_local(struct connman_ipconfig *ipconfig, const char *address)
+{
+	if (ipconfig->address == NULL)
+		return;
+
+	g_free(ipconfig->address->local);
+	ipconfig->address->local = g_strdup(address);
+}
+
+const char *__connman_ipconfig_get_peer(struct connman_ipconfig *ipconfig)
+{
+	if (ipconfig->address == NULL)
+		return NULL;
+
+	return ipconfig->address->peer;
+}
+
+void __connman_ipconfig_set_peer(struct connman_ipconfig *ipconfig, const char *address)
+{
+	if (ipconfig->address == NULL)
+		return;
+
+	g_free(ipconfig->address->peer);
+	ipconfig->address->peer = g_strdup(address);
+}
+
+const char *__connman_ipconfig_get_broadcast(struct connman_ipconfig *ipconfig)
+{
+	if (ipconfig->address == NULL)
+		return NULL;
+
+	return ipconfig->address->broadcast;
+}
+
+void __connman_ipconfig_set_broadcast(struct connman_ipconfig *ipconfig, const char *broadcast)
+{
+	if (ipconfig->address == NULL)
+		return;
+
+	g_free(ipconfig->address->broadcast);
+	ipconfig->address->broadcast = g_strdup(broadcast);
+}
+
+const char *__connman_ipconfig_get_gateway(struct connman_ipconfig *ipconfig)
+{
+	if (ipconfig->address == NULL)
+		return NULL;
+
+	return ipconfig->address->gateway;
+}
+
+void __connman_ipconfig_set_gateway(struct connman_ipconfig *ipconfig, const char *gateway)
+{
+	struct connman_service *service;
+
+	if (ipconfig->address == NULL)
+		return;
+
+	service = __connman_service_lookup_from_index(ipconfig->index);
+	if (service != NULL)
+		__connman_connection_gateway_remove(service);
+
+	g_free(ipconfig->address->gateway);
+	ipconfig->address->gateway = g_strdup(gateway);
+
+	if (service != NULL && ipconfig->address->gateway != NULL) {
+		if (ipconfig->type == CONNMAN_IPCONFIG_TYPE_IPV6) {
+			__connman_connection_gateway_add(service,
+						NULL,
+						ipconfig->address->gateway,
+						ipconfig->address->peer);
+		} else if (ipconfig->type == CONNMAN_IPCONFIG_TYPE_IPV4) {
+			__connman_connection_gateway_add(service,
+						ipconfig->address->gateway,
+						NULL,
+						ipconfig->address->peer);
+		}
+	}
+}
+
+unsigned char __connman_ipconfig_get_prefixlen(struct connman_ipconfig *ipconfig)
+{
+	if (ipconfig->address == NULL)
+		return 0;
+
+	return ipconfig->address->prefixlen;
+}
+
+void __connman_ipconfig_set_prefixlen(struct connman_ipconfig *ipconfig, unsigned char prefixlen)
+{
+	if (ipconfig->address == NULL)
+		return;
+
+	ipconfig->address->prefixlen = prefixlen;
+}
+
 static struct connman_ipconfig *create_ipv6config(int index)
 {
 	struct connman_ipconfig *ipv6config;
@@ -919,6 +1096,7 @@ static struct connman_ipconfig *create_ipv6config(int index)
 	ipv6config->index = index;
 	ipv6config->type = CONNMAN_IPCONFIG_TYPE_IPV6;
 	ipv6config->method = CONNMAN_IPCONFIG_METHOD_AUTO;
+	ipv6config->ipv6_privacy_config = 0;
 
 	ipv6config->address = connman_ipaddress_alloc(AF_INET6);
 	if (ipv6config->address == NULL) {
@@ -1149,33 +1327,6 @@ void __connman_ipconfig_set_element_ipv6_gateway(
 		element->ipv6.gateway = ipconfig->address->gateway;
 }
 
-/*
- * FIXME: The element soulution should be removed in the future
- * Set IPv4 and IPv6 gateway
- */
-int __connman_ipconfig_set_gateway(struct connman_ipconfig *ipconfig,
-						struct connman_element *parent)
-{
-	struct connman_element *connection;
-
-	connection = connman_element_create(NULL);
-
-	DBG("ipconfig %p", ipconfig);
-
-	connection->type  = CONNMAN_ELEMENT_TYPE_CONNECTION;
-	connection->index = ipconfig->index;
-
-	if (ipconfig->type == CONNMAN_IPCONFIG_TYPE_IPV4)
-		connection->ipv4.gateway = ipconfig->address->gateway;
-	else if (ipconfig->type == CONNMAN_IPCONFIG_TYPE_IPV6)
-		connection->ipv6.gateway = ipconfig->address->gateway;
-
-	if (connman_element_register(connection, parent) < 0)
-		connman_element_unref(connection);
-
-	return 0;
-}
-
 int __connman_ipconfig_set_address(struct connman_ipconfig *ipconfig)
 {
 	DBG("");
@@ -1267,6 +1418,46 @@ const char *__connman_ipconfig_get_proxy_autoconfig(struct connman_ipconfig *ipc
 	return ipdevice->pac;
 }
 
+static void disable_ipv6(struct connman_ipconfig *ipconfig)
+{
+	struct connman_ipdevice *ipdevice;
+
+	DBG("");
+
+	ipdevice = g_hash_table_lookup(ipdevice_hash,
+					GINT_TO_POINTER(ipconfig->index));
+	if (ipdevice == NULL)
+		return;
+
+	set_ipv6_state(ipdevice->ifname, FALSE);
+}
+
+static void enable_ipv6(struct connman_ipconfig *ipconfig)
+{
+	struct connman_ipdevice *ipdevice;
+
+	DBG("");
+
+	ipdevice = g_hash_table_lookup(ipdevice_hash,
+					GINT_TO_POINTER(ipconfig->index));
+	if (ipdevice == NULL)
+		return;
+
+	if (ipconfig->method == CONNMAN_IPCONFIG_METHOD_AUTO)
+		set_ipv6_privacy(ipdevice->ifname,
+				ipconfig->ipv6_privacy_config);
+
+	set_ipv6_state(ipdevice->ifname, TRUE);
+}
+
+void __connman_ipconfig_disable_ipv6(struct connman_ipconfig *ipconfig)
+{
+	if (ipconfig == NULL || ipconfig->type != CONNMAN_IPCONFIG_TYPE_IPV6)
+		return;
+
+	disable_ipv6(ipconfig);
+}
+
 int __connman_ipconfig_enable(struct connman_ipconfig *ipconfig)
 {
 	struct connman_ipdevice *ipdevice;
@@ -1292,6 +1483,7 @@ int __connman_ipconfig_enable(struct connman_ipconfig *ipconfig)
 		if (ipdevice->config_ipv6 == ipconfig)
 			return -EALREADY;
 		type = CONNMAN_IPCONFIG_TYPE_IPV6;
+		enable_ipv6(ipconfig);
 	} else
 		return -EINVAL;
 
@@ -1375,6 +1567,10 @@ int __connman_ipconfig_disable(struct connman_ipconfig *ipconfig)
 	if (ipdevice->config_ipv6 == ipconfig) {
 		ipconfig_list = g_list_remove(ipconfig_list, ipconfig);
 
+		if (ipdevice->config_ipv6->method ==
+						CONNMAN_IPCONFIG_METHOD_AUTO)
+			disable_ipv6(ipdevice->config_ipv6);
+
 		connman_ipaddress_clear(ipdevice->config_ipv6->system);
 		connman_ipconfig_unref(ipdevice->config_ipv6);
 		ipdevice->config_ipv6 = NULL;
@@ -1420,6 +1616,30 @@ enum connman_ipconfig_method __connman_ipconfig_string2method(const char *method
 		return CONNMAN_IPCONFIG_METHOD_UNKNOWN;
 }
 
+static const char *privacy2string(int privacy)
+{
+	if (privacy <= 0)
+		return "disabled";
+	else if (privacy == 1)
+		return "enabled";
+	else if (privacy > 1)
+		return "prefered";
+
+	return "disabled";
+}
+
+static int string2privacy(const char *privacy)
+{
+	if (g_strcmp0(privacy, "disabled") == 0)
+		return 0;
+	else if (g_strcmp0(privacy, "enabled") == 0)
+		return 1;
+	else if (g_strcmp0(privacy, "prefered") == 0)
+		return 2;
+	else
+		return 0;
+}
+
 void __connman_ipconfig_append_ipv4(struct connman_ipconfig *ipconfig,
 							DBusMessageIter *iter)
 {
@@ -1462,7 +1682,7 @@ void __connman_ipconfig_append_ipv4(struct connman_ipconfig *ipconfig,
 void __connman_ipconfig_append_ipv6(struct connman_ipconfig *ipconfig,
 							DBusMessageIter *iter)
 {
-	const char *str;
+	const char *str, *privacy;
 
 	DBG("");
 
@@ -1489,12 +1709,16 @@ void __connman_ipconfig_append_ipv6(struct connman_ipconfig *ipconfig,
 	if (ipconfig->system->gateway != NULL)
 		connman_dbus_dict_append_basic(iter, "Gateway",
 				DBUS_TYPE_STRING, &ipconfig->system->gateway);
+
+	privacy = privacy2string(ipconfig->ipv6_privacy_config);
+	connman_dbus_dict_append_basic(iter, "Privacy",
+				DBUS_TYPE_STRING, &privacy);
 }
 
 void __connman_ipconfig_append_ipv6config(struct connman_ipconfig *ipconfig,
 							DBusMessageIter *iter)
 {
-	const char *str;
+	const char *str, *privacy;
 
 	DBG("");
 
@@ -1529,6 +1753,10 @@ void __connman_ipconfig_append_ipv6config(struct connman_ipconfig *ipconfig,
 	if (ipconfig->address->gateway != NULL)
 		connman_dbus_dict_append_basic(iter, "Gateway",
 				DBUS_TYPE_STRING, &ipconfig->address->gateway);
+
+	privacy = privacy2string(ipconfig->ipv6_privacy_config);
+	connman_dbus_dict_append_basic(iter, "Privacy",
+				DBUS_TYPE_STRING, &privacy);
 }
 
 void __connman_ipconfig_append_ipv4config(struct connman_ipconfig *ipconfig,
@@ -1578,41 +1806,13 @@ void __connman_ipconfig_append_ipv4config(struct connman_ipconfig *ipconfig,
 				DBUS_TYPE_STRING, &ipconfig->address->gateway);
 }
 
-static void disable_ipv6(struct connman_ipconfig *ipconfig)
-{
-	struct connman_ipdevice *ipdevice;
-
-	DBG("");
-
-	ipdevice = g_hash_table_lookup(ipdevice_hash,
-					GINT_TO_POINTER(ipconfig->index));
-	if (ipdevice == NULL)
-		return;
-
-	set_ipv6_state(ipdevice->ifname, FALSE);
-}
-
-static void enable_ipv6(struct connman_ipconfig *ipconfig)
-{
-	struct connman_ipdevice *ipdevice;
-
-	DBG("");
-
-	ipdevice = g_hash_table_lookup(ipdevice_hash,
-					GINT_TO_POINTER(ipconfig->index));
-	if (ipdevice == NULL)
-		return;
-
-	set_ipv6_state(ipdevice->ifname, TRUE);
-}
-
 int __connman_ipconfig_set_config(struct connman_ipconfig *ipconfig,
 							DBusMessageIter *array)
 {
 	enum connman_ipconfig_method method = CONNMAN_IPCONFIG_METHOD_UNKNOWN;
 	const char *address = NULL, *netmask = NULL, *gateway = NULL,
-			*prefix_length_string = NULL;
-	int prefix_length = 0;
+		*prefix_length_string = NULL, *privacy_string = NULL;
+	int prefix_length = 0, privacy = 0;
 	DBusMessageIter dict;
 
 	DBG("ipconfig %p", ipconfig);
@@ -1671,12 +1871,20 @@ int __connman_ipconfig_set_config(struct connman_ipconfig *ipconfig,
 				return -EINVAL;
 
 			dbus_message_iter_get_basic(&entry, &gateway);
+		} else if (g_str_equal(key, "Privacy") == TRUE) {
+			if (type != DBUS_TYPE_STRING)
+				return -EINVAL;
+
+			dbus_message_iter_get_basic(&entry, &privacy_string);
+			privacy = string2privacy(privacy_string);
 		}
 		dbus_message_iter_next(&dict);
 	}
 
-	DBG("method %d address %s netmask %s gateway %s prefix_length %d",
-			method, address, netmask, gateway, prefix_length);
+	DBG("method %d address %s netmask %s gateway %s prefix_length %d "
+		"privacy %s",
+		method, address, netmask, gateway, prefix_length,
+		privacy_string);
 
 	switch (method) {
 	case CONNMAN_IPCONFIG_METHOD_UNKNOWN:
@@ -1694,6 +1902,8 @@ int __connman_ipconfig_set_config(struct connman_ipconfig *ipconfig,
 			return -EINVAL;
 
 		ipconfig->method = method;
+		if (privacy_string != NULL)
+			ipconfig->ipv6_privacy_config = privacy;
 		enable_ipv6(ipconfig);
 		break;
 
@@ -1771,6 +1981,22 @@ int __connman_ipconfig_load(struct connman_ipconfig *ipconfig,
 	if (ipconfig->method == CONNMAN_IPCONFIG_METHOD_UNKNOWN)
 		ipconfig->method = CONNMAN_IPCONFIG_METHOD_OFF;
 
+	if (ipconfig->type == CONNMAN_IPCONFIG_TYPE_IPV6) {
+		if (ipconfig->method == CONNMAN_IPCONFIG_METHOD_AUTO ||
+			ipconfig->method == CONNMAN_IPCONFIG_METHOD_MANUAL) {
+			char *privacy;
+			char *pprefix = g_strdup_printf("%sprivacy", prefix);
+			privacy = g_key_file_get_string(keyfile, identifier,
+							pprefix, NULL);
+			ipconfig->ipv6_privacy_config = string2privacy(privacy);
+			g_free(pprefix);
+			g_free(privacy);
+
+			__connman_ipconfig_enable(ipconfig);
+			enable_ipv6(ipconfig);
+		}
+	}
+
 	g_free(method);
 	g_free(key);
 
@@ -1845,6 +2071,14 @@ int __connman_ipconfig_save(struct connman_ipconfig *ipconfig,
 			key, ipconfig->address->gateway);
 	g_free(key);
 
+	if (ipconfig->type == CONNMAN_IPCONFIG_TYPE_IPV6) {
+		const char *privacy;
+		privacy = privacy2string(ipconfig->ipv6_privacy_config);
+		key = g_strdup_printf("%sprivacy", prefix);
+		g_key_file_set_string(keyfile, identifier, key, privacy);
+		g_free(key);
+	}
+
 	return 0;
 }
 
